<?php
declare(strict_types=1);

namespace Inti\Filter\Container;

use Inti\Container\Exception\RuntimeException;
use Laminas\Filter\FilterChain;
use Laminas\Filter\FilterInterface;
use Laminas\Filter\FilterPluginManager;
use Psr\Container\ContainerInterface;
use ReflectionClass;
use ReflectionException;

/**
 * Class FilterChainFactory
 *
 * @package Inti\Filter\Container
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-filter
 */
final class FilterChainFactory
{
    /**
     * @param ContainerInterface $container
     * @param string $requestedName
     * @param array|null $options
     * @return FilterInterface
     * @throws ReflectionException
     */
    public function __invoke(ContainerInterface $container, string $requestedName, ?array $options = null): FilterChain
    {
        /** @var FilterPluginManager $plugins */
        $plugins = $container->get(FilterPluginManager::class);

        $refl = new ReflectionClass($requestedName);

        if (!$refl->isSubclassOf(FilterChain::class)) {
            throw new RuntimeException(sprintf(
                "%s: '%s' does not extend '%s'.",
                __METHOD__,
                $refl->getName(),
                FilterChain::class
            ));
        }

        /** @var FilterChain $chain */
        $chain = $refl->newInstance($options);
        $chain->setPluginManager($plugins);

        return $chain;
    }
}
